"""
Created on Wed Jul  1 15:22:11 2020

@author: rdvan, shikha

This is the model by Beretta and Kuang, with equation for toxins and resistance.
Parameters are taken from literature
This case considers resisitance is not already present.

"""


import scipy.integrate as spi
import matplotlib.pyplot as plt
from model1_parameters import *

def system_resistance(t,y,p):
    # Define dependent variables

    X_s, X_i, P, T1,T2,X_r  = y[0], y[1], y[2],y[3], y[4],y[5]
    # X_s = susceptible cell concentration (number/ml)
    # X_i = infected cell concentration (number/ml)
    # P = phage concentration (number/ml)
    # X_r = resistant cell concentration (number/ml)
    
    mu, C, ep, Lambda, gamma, beta, eta1, alpha, eta2, psi = p[0], p[1], p[2], p[3], p[4], p[5], p[6], p[7], p[8],p[9]  
    # mu  = bacterial growthrate 1/h
    # C = carrying capacity 1/ml
    # ep = bacterial contact rate constant ml/h
    # Lambda = bacterial lysis death rate 1/h
    # gamma = phage deathrate 1/h
    # beta = virus replication factor
    # eta = toxin degradation rate 1/h
    # alpha = toxin produced per cell 
    # psi rate (1/h) of formation of a non suseptible cell 
    
    # total bacterial concentration
    X_t = X_s +X_i+ X_r                         
    
    # Define ODEs
    if X_s > 0:        
        dX_s = mu*X_s*(1 -X_t/C) - ep*X_s*P
    else:
        dX_s = 0
        X_s = 0
    dX_i = ep*X_s*P - Lambda*X_i
    dP = -ep*X_s*P - gamma*P + beta*Lambda*X_i
    dT1 = -eta1*T1 + alpha*Lambda*X_i
    dT2 = -eta2*T2 + alpha*Lambda*X_i
    dX_r = mu*X_r*(1 -X_t/C) + X_s*psi
    
    return [dX_s, dX_i, dP,dT1,dT2, dX_r]

if __name__ == "__main__":
    tend = 10                        # hours
    tspan = np.linspace(t0,tend,num=100)
    
    sol = spi.solve_ivp(lambda t, y: system_resistance(t,y,param_gut), [t0,tend], [X_s0_gut, X_i0_gut, P0_gut, T10_gut,T20_gut, X_r_gut], method='RK45', t_eval=tspan)
    
    
    # Plot the concentrations in time
    
    plot = plt.figure(1,figsize=(1000/96, 750/96), dpi=96)
    plot.clf()
    
    ax1 = plt.subplot(321)
    plt.plot(sol.t, sol.y[0], '-b')
    plt.xlabel('time (hours)', fontsize=11)
    plt.ylabel('$X_s$ (number/ml)', fontsize=11)
    ax1.set_title(r"a.", pad=20, fontsize=17, x=-.2)
    
    ax2 = plt.subplot(322)
    plt.plot(sol.t, sol.y[1], '-b')
    plt.xlabel('time (hours)', fontsize=11)
    plt.ylabel('$X_i$ (number/ml)', fontsize=11)
    ax2.set_title(r"b.", pad=20, fontsize=17, x=-.2)
    
    ax3 = plt.subplot(323)
    plt.plot(sol.t, sol.y[5], '-b')
    plt.xlabel('time (hours)', fontsize=11)
    plt.ylabel('$X_r$ (number/ml)', fontsize=11)
    ax3.set_title(r"c.", pad=20, fontsize=17, x=-.2)
    
    ax4 = plt.subplot(324)
    plt.plot(sol.t, sol.y[2], '-b')
    plt.xlabel('time (hours)', fontsize=11)
    plt.ylabel('P (number/ml)', fontsize=11)
    ax4.set_title(r"d.", pad=20, fontsize=17, x=-.2)
     
    ax5 = plt.subplot(325)
    plt.plot(sol.t, sol.y[3]*mass_T, '-b')
    plt.xlabel('time (hours)', fontsize=11)
    plt.ylabel('$T_C$ (\u03BCg/ml)', fontsize=11)
    ax5.set_title(r"e.", pad=20, fontsize=17, x=-.2)

    ax6 = plt.subplot(326)
    plt.plot(sol.t, sol.y[4], '-b')
    plt.xlabel('time (hours)', fontsize=11)
    plt.ylabel('$T_R$ (number/ml)', fontsize=11)
    ax6.set_title(r"f.", pad=20, fontsize=17, x=-.2)       
    
    plt.subplots_adjust(wspace=0.4, hspace=0.4)
    plt.savefig("Model 1 gut resistance case 1", bbox_inches='tight')
    plt.show()
    